#include <assert.h>

#include <aterm2.h>
#include <deprecated.h>
#include "list.h"

/*{{{  conversion functions */

ATerm stringToChars(const char *str)
{
  int len = strlen(str);
  int i;
  ATermList result = ATempty;

  for (i = len - 1; i >= 0; i--) {
    result = ATinsert(result, (ATerm) ATmakeInt(str[i]));
  }

  return (ATerm) result;
}

ATerm byteToChar(char ch)
{
    return (ATerm) ATmakeInt(ch);
}

char *charsToString(ATerm arg)
{
  ATermList list = (ATermList) arg;
  int len = ATgetLength(list);
  int i;
  char *str;

  str = (char *) malloc(len+1);
  if (str == NULL) {
      return NULL;
  }

  for (i = 0; !ATisEmpty(list); list = ATgetNext(list), i++) {
    str[i] = (char) ATgetInt((ATermInt) ATgetFirst(list));
  }
  str[i] = '\0';

  return str;
}

char charToByte(ATerm arg)
{
    return (char) ATgetInt((ATermInt) arg);
}


/*}}}  */

/*{{{  typedefs */

typedef struct ATerm _Module;
typedef struct ATerm _Modules;
typedef struct ATerm _Modules2;
typedef struct ATerm _ModuleList;
typedef struct ATerm _Separated;
typedef struct ATerm _Layout;
typedef struct ATerm _NineSeps;

/*}}}  */

/*{{{  void initListApi(void) */

void initListApi(void)
{
  init_list_dict();
}

/*}}}  */

/*{{{  (un)protect functions */

void protectModule(Module *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void unprotectModule(Module *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void protectModules(Modules *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void unprotectModules(Modules *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void protectModules2(Modules2 *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void unprotectModules2(Modules2 *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void protectModuleList(ModuleList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void unprotectModuleList(ModuleList *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void protectSeparated(Separated *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void unprotectSeparated(Separated *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void protectLayout(Layout *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void unprotectLayout(Layout *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void protectNineSeps(NineSeps *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void unprotectNineSeps(NineSeps *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}


/*}}}  */
/*{{{  term conversion functions */

/*{{{  Module ModuleFromTerm(ATerm t) */

Module ModuleFromTerm(ATerm t)
{
  return (Module)t;
}

/*}}}  */
/*{{{  ATerm ModuleToTerm(Module arg) */

ATerm ModuleToTerm(Module arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  Modules ModulesFromTerm(ATerm t) */

Modules ModulesFromTerm(ATerm t)
{
  return (Modules)t;
}

/*}}}  */
/*{{{  ATerm ModulesToTerm(Modules arg) */

ATerm ModulesToTerm(Modules arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  Modules2 Modules2FromTerm(ATerm t) */

Modules2 Modules2FromTerm(ATerm t)
{
  return (Modules2)t;
}

/*}}}  */
/*{{{  ATerm Modules2ToTerm(Modules2 arg) */

ATerm Modules2ToTerm(Modules2 arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  ModuleList ModuleListFromTerm(ATerm t) */

ModuleList ModuleListFromTerm(ATerm t)
{
  return (ModuleList)t;
}

/*}}}  */
/*{{{  ATerm ModuleListToTerm(ModuleList arg) */

ATerm ModuleListToTerm(ModuleList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  Separated SeparatedFromTerm(ATerm t) */

Separated SeparatedFromTerm(ATerm t)
{
  return (Separated)t;
}

/*}}}  */
/*{{{  ATerm SeparatedToTerm(Separated arg) */

ATerm SeparatedToTerm(Separated arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  Layout LayoutFromTerm(ATerm t) */

Layout LayoutFromTerm(ATerm t)
{
  return (Layout)t;
}

/*}}}  */
/*{{{  ATerm LayoutToTerm(Layout arg) */

ATerm LayoutToTerm(Layout arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  NineSeps NineSepsFromTerm(ATerm t) */

NineSeps NineSepsFromTerm(ATerm t)
{
  return (NineSeps)t;
}

/*}}}  */
/*{{{  ATerm NineSepsToTerm(NineSeps arg) */

ATerm NineSepsToTerm(NineSeps arg)
{
  return (ATerm)arg;
}

/*}}}  */

/*}}}  */
/*{{{  list functions */

int getModulesLength (Modules arg) {
  return ATgetLength((ATermList) arg);
}
Modules reverseModules(Modules arg) {
  return (Modules) ATreverse((ATermList) arg);
}
Modules appendModules(Modules arg, Module elem) {
  return (Modules) ATappend((ATermList) arg, (ATerm) ((ATerm) elem));
}
Modules concatModules(Modules arg0, Modules arg1) {
  return (Modules) ATconcat((ATermList) arg0, (ATermList) arg1);
}
Modules sliceModules(Modules arg, int start, int end) {
  return (Modules) ATgetSlice((ATermList) arg, start, end);
}
Module getModulesModuleAt(Modules arg, int index) {
 return (Module)ATelementAt((ATermList) arg,index);
}
Modules replaceModulesModuleAt(Modules arg, Module elem, int index) {
 return (Modules) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index);
}
Modules makeModules2(Module elem1, Module elem2) {
  return (Modules) ATmakeList2((ATerm) ((ATerm) elem2), (ATerm) ((ATerm) elem2));
}
Modules makeModules3(Module elem1, Module elem2, Module elem3) {
  return (Modules) ATmakeList3((ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3));
}
Modules makeModules4(Module elem1, Module elem2, Module elem3, Module elem4) {
  return (Modules) ATmakeList4((ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4));
}
Modules makeModules5(Module elem1, Module elem2, Module elem3, Module elem4, Module elem5) {
  return (Modules) ATmakeList5((ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5));
}
Modules makeModules6(Module elem1, Module elem2, Module elem3, Module elem4, Module elem5, Module elem6) {
  return (Modules) ATmakeList6((ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6));
}
int getModules2Length (Modules2 arg) {
  return ATgetLength((ATermList) arg);
}
Modules2 reverseModules2(Modules2 arg) {
  return (Modules2) ATreverse((ATermList) arg);
}
Modules2 appendModules2(Modules2 arg, Module elem) {
  return (Modules2) ATappend((ATermList) arg, (ATerm) ((ATerm) elem));
}
Modules2 concatModules2(Modules2 arg0, Modules2 arg1) {
  return (Modules2) ATconcat((ATermList) arg0, (ATermList) arg1);
}
Modules2 sliceModules2(Modules2 arg, int start, int end) {
  return (Modules2) ATgetSlice((ATermList) arg, start, end);
}
Module getModules2ModuleAt(Modules2 arg, int index) {
 return (Module)ATelementAt((ATermList) arg,index);
}
Modules2 replaceModules2ModuleAt(Modules2 arg, Module elem, int index) {
 return (Modules2) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index);
}
Modules2 makeModules22(Module elem1, Module elem2) {
  return (Modules2) ATmakeList2((ATerm) ((ATerm) elem2), (ATerm) ((ATerm) elem2));
}
Modules2 makeModules23(Module elem1, Module elem2, Module elem3) {
  return (Modules2) ATmakeList3((ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3));
}
Modules2 makeModules24(Module elem1, Module elem2, Module elem3, Module elem4) {
  return (Modules2) ATmakeList4((ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4));
}
Modules2 makeModules25(Module elem1, Module elem2, Module elem3, Module elem4, Module elem5) {
  return (Modules2) ATmakeList5((ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5));
}
Modules2 makeModules26(Module elem1, Module elem2, Module elem3, Module elem4, Module elem5, Module elem6) {
  return (Modules2) ATmakeList6((ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6));
}
int getSeparatedLength (Separated arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 4) + 1;
}
Separated reverseSeparated(Separated arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATerm sep2;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep2);
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (Separated) result;
}
Separated appendSeparated(Separated arg0, Layout wsl, Layout wsr, Module arg1) {
  return concatSeparated(arg0, wsl, wsr, makeSeparatedSingle(arg1));
}
Separated concatSeparated(Separated arg0, Layout wsl, Layout wsr, Separated arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = makeSeparatedMany((Module)ATgetFirst((ATermList) arg0), wsl, wsr,  arg1);
  arg1 = (Separated) ATgetNext((ATermList) arg1);
  return (Separated) ATconcat((ATermList) arg0, (ATermList) arg1);
}
Separated sliceSeparated(Separated arg, int start, int end) {
  return (Separated) ATgetSlice((ATermList) arg, start * 4, end * 4);
}
Module getSeparatedModuleAt(Separated arg, int index) {
 return (Module)ATelementAt((ATermList) arg,index * 4);
}
Separated replaceSeparatedModuleAt(Separated arg, Module elem, int index) {
 return (Separated) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 4);
}
Separated makeSeparated2(Layout wsl, Layout wsr, Module elem1, Module elem2) {
  return makeSeparatedMany(elem1, wsl, wsr, makeSeparatedSingle(elem2));
}
Separated makeSeparated3(Layout wsl, Layout wsr, Module elem1, Module elem2, Module elem3) {
  return makeSeparatedMany(elem1, wsl, wsr, makeSeparated2(wsl, wsr, elem2, elem3));
}
Separated makeSeparated4(Layout wsl, Layout wsr, Module elem1, Module elem2, Module elem3, Module elem4) {
  return makeSeparatedMany(elem1, wsl, wsr, makeSeparated3(wsl, wsr, elem2, elem3, elem4));
}
Separated makeSeparated5(Layout wsl, Layout wsr, Module elem1, Module elem2, Module elem3, Module elem4, Module elem5) {
  return makeSeparatedMany(elem1, wsl, wsr, makeSeparated4(wsl, wsr, elem2, elem3, elem4, elem5));
}
Separated makeSeparated6(Layout wsl, Layout wsr, Module elem1, Module elem2, Module elem3, Module elem4, Module elem5, Module elem6) {
  return makeSeparatedMany(elem1, wsl, wsr, makeSeparated5(wsl, wsr, elem2, elem3, elem4, elem5, elem6));
}
int getNineSepsLength (NineSeps arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 10) + 1;
}
NineSeps reverseNineSeps(NineSeps arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATerm sep2;
  ATerm sep3;
  ATerm sep4;
  ATerm sep5;
  ATerm sep6;
  ATerm sep7;
  ATerm sep8;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);
  sep3 = ATgetFirst(list);
  list = ATgetNext(list);
  sep4 = ATgetFirst(list);
  list = ATgetNext(list);
  sep5 = ATgetFirst(list);
  list = ATgetNext(list);
  sep6 = ATgetFirst(list);
  list = ATgetNext(list);
  sep7 = ATgetFirst(list);
  list = ATgetNext(list);
  sep8 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep8);
    result = ATinsert(result, sep7);
    result = ATinsert(result, sep6);
    result = ATinsert(result, sep5);
    result = ATinsert(result, sep4);
    result = ATinsert(result, sep3);
    result = ATinsert(result, sep2);
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);
  sep3 = ATgetFirst(list);
  list = ATgetNext(list);
  sep4 = ATgetFirst(list);
  list = ATgetNext(list);
  sep5 = ATgetFirst(list);
  list = ATgetNext(list);
  sep6 = ATgetFirst(list);
  list = ATgetNext(list);
  sep7 = ATgetFirst(list);
  list = ATgetNext(list);
  sep8 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (NineSeps) result;
}
NineSeps appendNineSeps(NineSeps arg0, Module arg1) {
  return concatNineSeps(arg0, makeNineSepsSingle(arg1));
}
NineSeps concatNineSeps(NineSeps arg0, NineSeps arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = makeNineSepsMany((Module)ATgetFirst((ATermList) arg0),  arg1);
  arg1 = (NineSeps) ATgetNext((ATermList) arg1);
  return (NineSeps) ATconcat((ATermList) arg0, (ATermList) arg1);
}
NineSeps sliceNineSeps(NineSeps arg, int start, int end) {
  return (NineSeps) ATgetSlice((ATermList) arg, start * 10, end * 10);
}
Module getNineSepsModuleAt(NineSeps arg, int index) {
 return (Module)ATelementAt((ATermList) arg,index * 10);
}
NineSeps replaceNineSepsModuleAt(NineSeps arg, Module elem, int index) {
 return (NineSeps) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 10);
}
NineSeps makeNineSeps2(Module elem1, Module elem2) {
  return makeNineSepsMany(elem1, makeNineSepsSingle(elem2));
}
NineSeps makeNineSeps3(Module elem1, Module elem2, Module elem3) {
  return makeNineSepsMany(elem1, makeNineSeps2(elem2, elem3));
}
NineSeps makeNineSeps4(Module elem1, Module elem2, Module elem3, Module elem4) {
  return makeNineSepsMany(elem1, makeNineSeps3(elem2, elem3, elem4));
}
NineSeps makeNineSeps5(Module elem1, Module elem2, Module elem3, Module elem4, Module elem5) {
  return makeNineSepsMany(elem1, makeNineSeps4(elem2, elem3, elem4, elem5));
}
NineSeps makeNineSeps6(Module elem1, Module elem2, Module elem3, Module elem4, Module elem5, Module elem6) {
  return makeNineSepsMany(elem1, makeNineSeps5(elem2, elem3, elem4, elem5, elem6));
}

/*}}}  */
/*{{{  constructors */

/*{{{  Module makeModuleDefault(const char* string) */

Module makeModuleDefault(const char* string)
{
  return (Module)(ATerm) (ATerm) ATmakeAppl(ATmakeAFun(string, 0, ATtrue));
}

/*}}}  */
/*{{{  Modules makeModulesEmpty(void) */

Modules makeModulesEmpty(void)
{
  return (Modules)(ATerm)ATempty;
}

/*}}}  */
/*{{{  Modules makeModulesSingle(Module head) */

Modules makeModulesSingle(Module head)
{
  return (Modules)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  Modules makeModulesMany(Module head, Modules tail) */

Modules makeModulesMany(Module head, Modules tail)
{
  return (Modules)(ATerm)ATinsert((ATermList)tail, (ATerm) head);
}

/*}}}  */
/*{{{  Modules2 makeModules2Empty(void) */

Modules2 makeModules2Empty(void)
{
  return (Modules2)(ATerm)ATempty;
}

/*}}}  */
/*{{{  Modules2 makeModules2Single(Module head) */

Modules2 makeModules2Single(Module head)
{
  return (Modules2)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  Modules2 makeModules2Many(Module head, Modules2 tail) */

Modules2 makeModules2Many(Module head, Modules2 tail)
{
  return (Modules2)(ATerm)ATinsert((ATermList)tail, (ATerm) head);
}

/*}}}  */
/*{{{  ModuleList makeModuleListModules(Modules modules) */

ModuleList makeModuleListModules(Modules modules)
{
  return (ModuleList)(ATerm)ATmakeAppl1(afun0, (ATerm) modules);
}

/*}}}  */
/*{{{  Separated makeSeparatedEmpty(void) */

Separated makeSeparatedEmpty(void)
{
  return (Separated)(ATerm)ATempty;
}

/*}}}  */
/*{{{  Separated makeSeparatedSingle(Module head) */

Separated makeSeparatedSingle(Module head)
{
  return (Separated)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  Separated makeSeparatedMany(Module head, Layout wsl, Layout wsr, Separated tail) */

Separated makeSeparatedMany(Module head, Layout wsl, Layout wsr, Separated tail)
{
  if (isSeparatedEmpty(tail)) {
    return makeSeparatedSingle(head);
  }
  return (Separated)(ATerm)ATinsert(ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm)ATmakeAppl1(afun1, (ATerm) wsr)), (ATerm)ATmakeAppl0(afun2)), (ATerm)ATmakeAppl1(afun1, (ATerm) wsl)), (ATerm) head);
}

/*}}}  */
/*{{{  Layout makeLayoutDefault(const char* string) */

Layout makeLayoutDefault(const char* string)
{
  return (Layout)(ATerm) (ATerm) ATmakeAppl(ATmakeAFun(string, 0, ATtrue));
}

/*}}}  */
/*{{{  NineSeps makeNineSepsEmpty(void) */

NineSeps makeNineSepsEmpty(void)
{
  return (NineSeps)(ATerm)ATempty;
}

/*}}}  */
/*{{{  NineSeps makeNineSepsSingle(Module head) */

NineSeps makeNineSepsSingle(Module head)
{
  return (NineSeps)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  NineSeps makeNineSepsMany(Module head, NineSeps tail) */

NineSeps makeNineSepsMany(Module head, NineSeps tail)
{
  if (isNineSepsEmpty(tail)) {
    return makeNineSepsSingle(head);
  }
  return (NineSeps)(ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm)ATmakeInt(9)), (ATerm)ATmakeInt(8)), (ATerm)ATmakeInt(7)), (ATerm)ATmakeInt(6)), (ATerm)ATmakeInt(5)), (ATerm)ATmakeInt(4)), (ATerm)ATmakeInt(3)), (ATerm)ATmakeInt(2)), (ATerm)ATmakeInt(1)), (ATerm) head);
}

/*}}}  */

/*}}}  */
/*{{{  equality functions */

ATbool isEqualModule(Module arg0, Module arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool isEqualModules(Modules arg0, Modules arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool isEqualModules2(Modules2 arg0, Modules2 arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool isEqualModuleList(ModuleList arg0, ModuleList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool isEqualSeparated(Separated arg0, Separated arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool isEqualLayout(Layout arg0, Layout arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool isEqualNineSeps(NineSeps arg0, NineSeps arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

/*}}}  */
/*{{{  Module accessors */

/*{{{  ATbool isValidModule(Module arg) */

ATbool isValidModule(Module arg)
{
  if (isModuleDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isModuleDefault(Module arg) */

inline ATbool isModuleDefault(Module arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternModuleDefault, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool hasModuleString(Module arg) */

ATbool hasModuleString(Module arg)
{
  if (isModuleDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char* getModuleString(Module arg) */

char* getModuleString(Module arg)
{
  
    return (char*)ATgetName(ATgetAFun((ATermAppl) arg));
}

/*}}}  */
/*{{{  Module setModuleString(Module arg, const char* string) */

Module setModuleString(Module arg, const char* string)
{
  if (isModuleDefault(arg)) {
    return (Module)((ATerm) (ATerm) ATmakeAppl(ATmakeAFun(string, 0, ATtrue)));
  }

  ATabort("Module has no String: %t\n", arg);
  return (Module)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  Modules accessors */

/*{{{  ATbool isValidModules(Modules arg) */

ATbool isValidModules(Modules arg)
{
  if (isModulesEmpty(arg)) {
    return ATtrue;
  }
  else if (isModulesSingle(arg)) {
    return ATtrue;
  }
  else if (isModulesMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isModulesEmpty(Modules arg) */

inline ATbool isModulesEmpty(Modules arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternModulesEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool isModulesSingle(Modules arg) */

inline ATbool isModulesSingle(Modules arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternModulesSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isModulesMany(Modules arg) */

inline ATbool isModulesMany(Modules arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternModulesMany, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool hasModulesHead(Modules arg) */

ATbool hasModulesHead(Modules arg)
{
  if (isModulesSingle(arg)) {
    return ATtrue;
  }
  else if (isModulesMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasModulesTail(Modules arg) */

ATbool hasModulesTail(Modules arg)
{
  if (isModulesMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  Module getModulesHead(Modules arg) */

Module getModulesHead(Modules arg)
{
  if (isModulesSingle(arg)) {
    return (Module)ATgetFirst((ATermList)arg);
  }
  else 
    return (Module)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  Modules getModulesTail(Modules arg) */

Modules getModulesTail(Modules arg)
{
  
    return (Modules)ATgetNext((ATermList)arg);
}

/*}}}  */
/*{{{  Modules setModulesHead(Modules arg, Module head) */

Modules setModulesHead(Modules arg, Module head)
{
  if (isModulesSingle(arg)) {
    return (Modules)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (isModulesMany(arg)) {
    return (Modules)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("Modules has no Head: %t\n", arg);
  return (Modules)NULL;
}

/*}}}  */
/*{{{  Modules setModulesTail(Modules arg, Modules tail) */

Modules setModulesTail(Modules arg, Modules tail)
{
  if (isModulesMany(arg)) {
    return (Modules)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 1);
  }

  ATabort("Modules has no Tail: %t\n", arg);
  return (Modules)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  Modules2 accessors */

/*{{{  ATbool isValidModules2(Modules2 arg) */

ATbool isValidModules2(Modules2 arg)
{
  if (isModules2Empty(arg)) {
    return ATtrue;
  }
  else if (isModules2Single(arg)) {
    return ATtrue;
  }
  else if (isModules2Many(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isModules2Empty(Modules2 arg) */

inline ATbool isModules2Empty(Modules2 arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternModules2Empty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool isModules2Single(Modules2 arg) */

inline ATbool isModules2Single(Modules2 arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternModules2Single, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isModules2Many(Modules2 arg) */

inline ATbool isModules2Many(Modules2 arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternModules2Many, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool hasModules2Head(Modules2 arg) */

ATbool hasModules2Head(Modules2 arg)
{
  if (isModules2Single(arg)) {
    return ATtrue;
  }
  else if (isModules2Many(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasModules2Tail(Modules2 arg) */

ATbool hasModules2Tail(Modules2 arg)
{
  if (isModules2Many(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  Module getModules2Head(Modules2 arg) */

Module getModules2Head(Modules2 arg)
{
  if (isModules2Single(arg)) {
    return (Module)ATgetFirst((ATermList)arg);
  }
  else 
    return (Module)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  Modules2 getModules2Tail(Modules2 arg) */

Modules2 getModules2Tail(Modules2 arg)
{
  
    return (Modules2)ATgetNext((ATermList)arg);
}

/*}}}  */
/*{{{  Modules2 setModules2Head(Modules2 arg, Module head) */

Modules2 setModules2Head(Modules2 arg, Module head)
{
  if (isModules2Single(arg)) {
    return (Modules2)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (isModules2Many(arg)) {
    return (Modules2)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("Modules2 has no Head: %t\n", arg);
  return (Modules2)NULL;
}

/*}}}  */
/*{{{  Modules2 setModules2Tail(Modules2 arg, Modules2 tail) */

Modules2 setModules2Tail(Modules2 arg, Modules2 tail)
{
  if (isModules2Many(arg)) {
    return (Modules2)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 1);
  }

  ATabort("Modules2 has no Tail: %t\n", arg);
  return (Modules2)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  ModuleList accessors */

/*{{{  ATbool isValidModuleList(ModuleList arg) */

ATbool isValidModuleList(ModuleList arg)
{
  if (isModuleListModules(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isModuleListModules(ModuleList arg) */

inline ATbool isModuleListModules(ModuleList arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternModuleListModules, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool hasModuleListModules(ModuleList arg) */

ATbool hasModuleListModules(ModuleList arg)
{
  if (isModuleListModules(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  Modules getModuleListModules(ModuleList arg) */

Modules getModuleListModules(ModuleList arg)
{
  
    return (Modules)ATgetArgument((ATermAppl)arg, 0);
}

/*}}}  */
/*{{{  ModuleList setModuleListModules(ModuleList arg, Modules modules) */

ModuleList setModuleListModules(ModuleList arg, Modules modules)
{
  if (isModuleListModules(arg)) {
    return (ModuleList)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) modules), 0);
  }

  ATabort("ModuleList has no Modules: %t\n", arg);
  return (ModuleList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  Separated accessors */

/*{{{  ATbool isValidSeparated(Separated arg) */

ATbool isValidSeparated(Separated arg)
{
  if (isSeparatedEmpty(arg)) {
    return ATtrue;
  }
  else if (isSeparatedSingle(arg)) {
    return ATtrue;
  }
  else if (isSeparatedMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isSeparatedEmpty(Separated arg) */

inline ATbool isSeparatedEmpty(Separated arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternSeparatedEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool isSeparatedSingle(Separated arg) */

inline ATbool isSeparatedSingle(Separated arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternSeparatedSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isSeparatedMany(Separated arg) */

inline ATbool isSeparatedMany(Separated arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternSeparatedMany, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool hasSeparatedHead(Separated arg) */

ATbool hasSeparatedHead(Separated arg)
{
  if (isSeparatedSingle(arg)) {
    return ATtrue;
  }
  else if (isSeparatedMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasSeparatedWsl(Separated arg) */

ATbool hasSeparatedWsl(Separated arg)
{
  if (isSeparatedMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasSeparatedWsr(Separated arg) */

ATbool hasSeparatedWsr(Separated arg)
{
  if (isSeparatedMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasSeparatedTail(Separated arg) */

ATbool hasSeparatedTail(Separated arg)
{
  if (isSeparatedMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  Separated getSeparatedTail(Separated arg) */

Separated getSeparatedTail(Separated arg)
{
  assert(!isSeparatedEmpty(arg) && "getTail on an empty list");
  if (isSeparatedSingle(arg)) {
    return (Separated) makeSeparatedEmpty();
  }
  else {
  
    return (Separated)ATgetTail((ATermList)arg, 4);
  }
}

/*}}}  */
/*{{{  Module getSeparatedHead(Separated arg) */

Module getSeparatedHead(Separated arg)
{
  if (isSeparatedSingle(arg)) {
    return (Module)ATgetFirst((ATermList)arg);
  }
  else 
    return (Module)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  Layout getSeparatedWsl(Separated arg) */

Layout getSeparatedWsl(Separated arg)
{
  
    return (Layout)ATgetArgument((ATermAppl)ATelementAt((ATermList)arg, 1), 0);
}

/*}}}  */
/*{{{  Layout getSeparatedWsr(Separated arg) */

Layout getSeparatedWsr(Separated arg)
{
  
    return (Layout)ATgetArgument((ATermAppl)ATelementAt((ATermList)arg, 3), 0);
}

/*}}}  */
/*{{{  Separated setSeparatedHead(Separated arg, Module head) */

Separated setSeparatedHead(Separated arg, Module head)
{
  if (isSeparatedSingle(arg)) {
    return (Separated)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (isSeparatedMany(arg)) {
    return (Separated)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("Separated has no Head: %t\n", arg);
  return (Separated)NULL;
}

/*}}}  */
/*{{{  Separated setSeparatedWsl(Separated arg, Layout wsl) */

Separated setSeparatedWsl(Separated arg, Layout wsl)
{
  if (isSeparatedMany(arg)) {
    return (Separated)ATreplace((ATermList)arg, (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)arg, 1), (ATerm)((ATerm) wsl), 0), 1);
  }

  ATabort("Separated has no Wsl: %t\n", arg);
  return (Separated)NULL;
}

/*}}}  */
/*{{{  Separated setSeparatedWsr(Separated arg, Layout wsr) */

Separated setSeparatedWsr(Separated arg, Layout wsr)
{
  if (isSeparatedMany(arg)) {
    return (Separated)ATreplace((ATermList)arg, (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)arg, 3), (ATerm)((ATerm) wsr), 0), 3);
  }

  ATabort("Separated has no Wsr: %t\n", arg);
  return (Separated)NULL;
}

/*}}}  */
/*{{{  Separated setSeparatedTail(Separated arg, Separated tail) */

Separated setSeparatedTail(Separated arg, Separated tail)
{
  if (isSeparatedMany(arg)) {
    return (Separated)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 4);
  }

  ATabort("Separated has no Tail: %t\n", arg);
  return (Separated)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  Layout accessors */

/*{{{  ATbool isValidLayout(Layout arg) */

ATbool isValidLayout(Layout arg)
{
  if (isLayoutDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isLayoutDefault(Layout arg) */

inline ATbool isLayoutDefault(Layout arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternLayoutDefault, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool hasLayoutString(Layout arg) */

ATbool hasLayoutString(Layout arg)
{
  if (isLayoutDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char* getLayoutString(Layout arg) */

char* getLayoutString(Layout arg)
{
  
    return (char*)ATgetName(ATgetAFun((ATermAppl) arg));
}

/*}}}  */
/*{{{  Layout setLayoutString(Layout arg, const char* string) */

Layout setLayoutString(Layout arg, const char* string)
{
  if (isLayoutDefault(arg)) {
    return (Layout)((ATerm) (ATerm) ATmakeAppl(ATmakeAFun(string, 0, ATtrue)));
  }

  ATabort("Layout has no String: %t\n", arg);
  return (Layout)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  NineSeps accessors */

/*{{{  ATbool isValidNineSeps(NineSeps arg) */

ATbool isValidNineSeps(NineSeps arg)
{
  if (isNineSepsEmpty(arg)) {
    return ATtrue;
  }
  else if (isNineSepsSingle(arg)) {
    return ATtrue;
  }
  else if (isNineSepsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isNineSepsEmpty(NineSeps arg) */

inline ATbool isNineSepsEmpty(NineSeps arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternNineSepsEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool isNineSepsSingle(NineSeps arg) */

inline ATbool isNineSepsSingle(NineSeps arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternNineSepsSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isNineSepsMany(NineSeps arg) */

inline ATbool isNineSepsMany(NineSeps arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternNineSepsMany, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool hasNineSepsHead(NineSeps arg) */

ATbool hasNineSepsHead(NineSeps arg)
{
  if (isNineSepsSingle(arg)) {
    return ATtrue;
  }
  else if (isNineSepsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasNineSepsTail(NineSeps arg) */

ATbool hasNineSepsTail(NineSeps arg)
{
  if (isNineSepsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  NineSeps getNineSepsTail(NineSeps arg) */

NineSeps getNineSepsTail(NineSeps arg)
{
  assert(!isNineSepsEmpty(arg) && "getTail on an empty list");
  if (isNineSepsSingle(arg)) {
    return (NineSeps) makeNineSepsEmpty();
  }
  else {
  
    return (NineSeps)ATgetTail((ATermList)arg, 10);
  }
}

/*}}}  */
/*{{{  Module getNineSepsHead(NineSeps arg) */

Module getNineSepsHead(NineSeps arg)
{
  if (isNineSepsSingle(arg)) {
    return (Module)ATgetFirst((ATermList)arg);
  }
  else 
    return (Module)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  NineSeps setNineSepsHead(NineSeps arg, Module head) */

NineSeps setNineSepsHead(NineSeps arg, Module head)
{
  if (isNineSepsSingle(arg)) {
    return (NineSeps)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (isNineSepsMany(arg)) {
    return (NineSeps)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("NineSeps has no Head: %t\n", arg);
  return (NineSeps)NULL;
}

/*}}}  */
/*{{{  NineSeps setNineSepsTail(NineSeps arg, NineSeps tail) */

NineSeps setNineSepsTail(NineSeps arg, NineSeps tail)
{
  if (isNineSepsMany(arg)) {
    return (NineSeps)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 10);
  }

  ATabort("NineSeps has no Tail: %t\n", arg);
  return (NineSeps)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  sort visitors */

/*{{{  Module visitModule(Module arg, char* (*acceptString)(char*)) */

Module visitModule(Module arg, char* (*acceptString)(char*))
{
  if (isModuleDefault(arg)) {
    return makeModuleDefault(
        acceptString ? acceptString(getModuleString(arg)) : getModuleString(arg));
  }
  ATabort("not a Module: %t\n", arg);
  return (Module)NULL;
}

/*}}}  */
/*{{{  Modules visitModules(Modules arg, Module (*acceptHead)(Module)) */

Modules visitModules(Modules arg, Module (*acceptHead)(Module))
{
  if (isModulesEmpty(arg)) {
    return makeModulesEmpty();
  }
  if (isModulesSingle(arg)) {
    return makeModulesSingle(
        acceptHead ? acceptHead(getModulesHead(arg)) : getModulesHead(arg));
  }
  if (isModulesMany(arg)) {
    return makeModulesMany(
        acceptHead ? acceptHead(getModulesHead(arg)) : getModulesHead(arg),
        visitModules(getModulesTail(arg), acceptHead));
  }
  ATabort("not a Modules: %t\n", arg);
  return (Modules)NULL;
}

/*}}}  */
/*{{{  Modules2 visitModules2(Modules2 arg, Module (*acceptHead)(Module)) */

Modules2 visitModules2(Modules2 arg, Module (*acceptHead)(Module))
{
  if (isModules2Empty(arg)) {
    return makeModules2Empty();
  }
  if (isModules2Single(arg)) {
    return makeModules2Single(
        acceptHead ? acceptHead(getModules2Head(arg)) : getModules2Head(arg));
  }
  if (isModules2Many(arg)) {
    return makeModules2Many(
        acceptHead ? acceptHead(getModules2Head(arg)) : getModules2Head(arg),
        visitModules2(getModules2Tail(arg), acceptHead));
  }
  ATabort("not a Modules2: %t\n", arg);
  return (Modules2)NULL;
}

/*}}}  */
/*{{{  ModuleList visitModuleList(ModuleList arg, Modules (*acceptModules)(Modules)) */

ModuleList visitModuleList(ModuleList arg, Modules (*acceptModules)(Modules))
{
  if (isModuleListModules(arg)) {
    return makeModuleListModules(
        acceptModules ? acceptModules(getModuleListModules(arg)) : getModuleListModules(arg));
  }
  ATabort("not a ModuleList: %t\n", arg);
  return (ModuleList)NULL;
}

/*}}}  */
/*{{{  Separated visitSeparated(Separated arg, Module (*acceptHead)(Module), Layout (*acceptWsl)(Layout), Layout (*acceptWsr)(Layout)) */

Separated visitSeparated(Separated arg, Module (*acceptHead)(Module), Layout (*acceptWsl)(Layout), Layout (*acceptWsr)(Layout))
{
  if (isSeparatedEmpty(arg)) {
    return makeSeparatedEmpty();
  }
  if (isSeparatedSingle(arg)) {
    return makeSeparatedSingle(
        acceptHead ? acceptHead(getSeparatedHead(arg)) : getSeparatedHead(arg));
  }
  if (isSeparatedMany(arg)) {
    return makeSeparatedMany(
        acceptHead ? acceptHead(getSeparatedHead(arg)) : getSeparatedHead(arg),
        acceptWsl ? acceptWsl(getSeparatedWsl(arg)) : getSeparatedWsl(arg),
        acceptWsr ? acceptWsr(getSeparatedWsr(arg)) : getSeparatedWsr(arg),
        visitSeparated(getSeparatedTail(arg), acceptHead, acceptWsl, acceptWsr));
  }
  ATabort("not a Separated: %t\n", arg);
  return (Separated)NULL;
}

/*}}}  */
/*{{{  Layout visitLayout(Layout arg, char* (*acceptString)(char*)) */

Layout visitLayout(Layout arg, char* (*acceptString)(char*))
{
  if (isLayoutDefault(arg)) {
    return makeLayoutDefault(
        acceptString ? acceptString(getLayoutString(arg)) : getLayoutString(arg));
  }
  ATabort("not a Layout: %t\n", arg);
  return (Layout)NULL;
}

/*}}}  */
/*{{{  NineSeps visitNineSeps(NineSeps arg, Module (*acceptHead)(Module)) */

NineSeps visitNineSeps(NineSeps arg, Module (*acceptHead)(Module))
{
  if (isNineSepsEmpty(arg)) {
    return makeNineSepsEmpty();
  }
  if (isNineSepsSingle(arg)) {
    return makeNineSepsSingle(
        acceptHead ? acceptHead(getNineSepsHead(arg)) : getNineSepsHead(arg));
  }
  if (isNineSepsMany(arg)) {
    return makeNineSepsMany(
        acceptHead ? acceptHead(getNineSepsHead(arg)) : getNineSepsHead(arg),
        visitNineSeps(getNineSepsTail(arg), acceptHead));
  }
  ATabort("not a NineSeps: %t\n", arg);
  return (NineSeps)NULL;
}

/*}}}  */

/*}}}  */
